from exposer.exposer import Exposer
from exposer.actions import PathAction
from exposer.targets import Wsgi
from exposer.actions import CliAction
from exposer.targets import Cli
from gunicornapp import RunApp


class app(object):
    cli = Exposer(action=CliAction, target=Cli)
    web = Exposer(action=PathAction, target=Wsgi)


class people:
    @staticmethod
    @app.web
    @app.cli
    def greetings(
        name=None, surname=None, greeting="Hello"
    ):
        return "{}, {} {}".format(
            greeting,
            name.capitalize(),
            surname.capitalize(),
        )


@app.web
@app.cli
def index():
    return "Welcome"


@app.cli
def run():
    APP = app.web.expose()
    app.web.rename("/index", "/")
    RunApp(APP)


app.cli.expose()



class BaseAction(object):
    starts_with = "<"
    method_seperator = "-"
    action_pointer = "@"
    instance_pointer = "#"
    ends_with = ">"

    def __init__(self, action):
        self.action = action

    def __call__(self, *args, **kwargs):
        return self.action(*args, **kwargs)

    def __repr__(self):
        hsh = ""
        if hasattr(self.action, "__qualname__"):
            hsh = (
                str(self.action_pointer)
                .join(self.action.__qualname__.split("."))
                .lower()
            )

        if hasattr(self.action, "__self__"):
            tmp = hsh.split(str(self.action_pointer))
            hsh = (
                str(self.action_pointer).join(
                    [
                        str(self.method_seperator).join(
                            tmp[0:-1]
                        ),
                        str(tmp[-1]),
                    ]
                )
                + str(self.instance_pointer)
                + str(self.action.__self__)
            )
        hsh = (
            str(self.starts_with)
            + hsh
            + str(self.ends_with)
        )
        return hsh


class Action(BaseAction):
    pass


class CliAction(BaseAction):
    starts_with = ""
    method_seperator = ":"
    action_pointer = ":"
    instance_pointer = ":"
    ends_with = ""


class PathAction(BaseAction):
    starts_with = "/"
    method_seperator = "/"
    action_pointer = "/"
    instance_pointer = "/"
    ends_with = ""



from .actions import CliAction
from .targets import Cli


class Encapsulate(object):
    value = None

    def __get__(self, instance, owner):
        return self.value

    def __set__(self, instance, value):
        self.value = value

    def __delete__(self, instance):
        del self


class Exposer(object):
    count = 0
    action = None
    name = None

    def __set_name__(self, owner, name):
        self.name = name
        self.help = self(self.help)

    def __init__(self, action=CliAction, target=Cli):
        Exposer.count += 1
        self.live_id = Exposer.count
        self.actions = {}
        self.action = action
        self.target = target

    def __repr__(self):
        return str(self.name)

    def rename(self, name, newname):
        action = self.actions.pop(name)
        self.actions[newname] = action

    def help(self):
        """show this output"""
        for kw, val in self.actions.items():
            print(kw)

    def __call__(self, function):
        action = self.action(function)
        self.actions[str(action)] = action
        return function

    def expose(self):
        self.app = self.target(self.actions)
        return self.app()



from __future__ import unicode_literals
import multiprocessing
import gunicorn.app.base
from gunicorn.six import iteritems


def number_of_workers():
    return (multiprocessing.cpu_count() * 2) + 1


def handler_app(environ, start_response):
    response_body = b"Works fine"
    status = "200 OK"

    response_headers = [("Content-Type", "text/plain")]

    start_response(status, response_headers)

    return [response_body]


class StandaloneApplication(
    gunicorn.app.base.BaseApplication
):
    def __init__(self, app, options=None):
        self.options = options or {}
        self.application = app
        super(StandaloneApplication, self).__init__()

    def load_config(self):
        config = dict(
            [
                (key, value)
                for key, value in iteritems(self.options)
                if key in self.cfg.settings
                and value is not None
            ]
        )
        for key, value in iteritems(config):
            self.cfg.set(key.lower(), value)

    def load(self):
        return self.application


def RunApp(handler_app):
    options = {
        "bind": "%s:%s" % ("127.0.0.1", "8080"),
        "workers": number_of_workers(),
    }
    StandaloneApplication(handler_app, options).run()



import re
import sys
import pprint


class Cli(object):
    def __init__(self, actions):
        self.actions = actions
        print(80 * "-")
        pprint.pprint(self.actions)
        print(80 * "-")
        self.argv = sys.argv

    def __call__(self):
        if len(self.argv) == 1:
            return 1
        else:
            if self.argv[1] in self.actions.keys():
                self.args = []
                self.kwargs = {}
                arg_flag = True
                for i in self.argv[2:]:
                    match = re.search(r"(\w*)=(.*)", i)
                    if match:
                        self.kwargs[
                            match.group(1)
                        ] = match.group(2)
                        arg_flag = False
                    else:
                        if arg_flag:
                            self.args.append(i)
                        else:
                            raise Exception(
                                "kwargs must placed end"
                            )
                result = self.actions[self.argv[1]](
                    *self.args, **self.kwargs
                )
                print(result)
            return None


class Wsgi(object):
    def __init__(self, actions):
        self.actions = actions
        pprint.pprint(self.actions)

    def __call__(self):
        def handler(environ, start_response):
            """Simplest possible application object"""
            status = "200 OK"
            response_headers = [
                ("Content-type", "text/plain")
            ]
            start_response(status, response_headers)

            result = ""
            if environ["PATH_INFO"] in self.actions.keys():
                self.args = []
                self.kwargs = {}
                arg_flag = True
                for i in environ["QUERY_STRING"].split(
                    "&"
                ):
                    match = re.search(r"(\w*)=(.*)", i)
                    if match:
                        self.kwargs[
                            match.group(1)
                        ] = match.group(2)
                        arg_flag = False
                    else:
                        if arg_flag:
                            self.args.append(i)
                        else:
                            raise Exception(
                                "kwargs must placed end"
                            )
                result = self.actions[
                    environ["PATH_INFO"]
                ](*self.args, **self.kwargs)
            return [str(result).encode()]
            return [
                "\n".encode().join(
                    [
                        kw.encode()
                        + str(":").encode()
                        + str(item).encode()
                        for kw, item in environ.items()
                    ]
                )
            ]

        return handler
